#include <iostream>

using namespace std;

// Film als Datenstruktur
class Film {
    friend class FilmListe;

   private:
    string titel; 
    unsigned int jahr; 
    string genre;
    Film *nachfolger;

   public:
    Film() {}
    Film(string t, int j, string g) : titel(t), jahr(j), genre(g) {}
};

// Basic structure
class FilmListe {
    // kopf
    Film *kopf;

    // ende
    Film *ende;

   public:
    // FilmListe
    FilmListe() { kopf = ende = NULL; }

    // Destruktor
    ~FilmListe() { cout << "Liste gelöscht" << endl; }

    // einen Film in die Liste einfügen
    void hinzufuegen(Film film) {
        // Ein neues Listenelement erstellen und mit 'film' initialisieren
        Film *neuesListenelement = new Film(film);

        // liste ist leer
        if (istLeer()) {
			ende = kopf = neuesListenelement;
			kopf->nachfolger = NULL;
			ende->nachfolger = NULL;
        }

        else {
			// das letzte Element zeigt auf das neue Element
			ende->nachfolger = neuesListenelement;

			// das neue Element wird zum Letzten
			ende = neuesListenelement;
			
			// der Nachfolger vom Ende ist NULL
			ende->nachfolger = NULL;
        }
    }

    // prüft ob die Liste leer ist
    bool istLeer() { return (kopf == NULL) ? true : false; }

    // Liste löschen
    void loeschen() {
        if (istLeer()) return;

        // solange der Zeiger nicht Null ist, also noch Elemente vorhanden sind...
        while(kopf->nachfolger != NULL) {
            // ...suche das vorletzte ELement
			Film *vorletztesElement = kopf;
			while(vorletztesElement->nachfolger != ende) {
				vorletztesElement = vorletztesElement->nachfolger;
			}

			// lösche das letzte Element
			delete ende;

			// das vorletzte Element wird zum Letzten
			vorletztesElement->nachfolger = NULL;
			ende = vorletztesElement;
        }

		// zuletzt noch den Listenkopf löschen
		delete kopf;
    }

	// zeigt alle Listenelemente
	void elementeAnzeigen() {
		// aktueller Knoten
		Film *p = kopf;

		// solange der Knoten nicht Null ist, also das Ende nicht erreicht ist...
		while(p != NULL) {
            // ...Inhalt ausgeben
			cout << "Titel: "<< p->titel
				 << "\tJahr: " << p->jahr 
				 << "\tGenre: " << p->genre << endl;

			// der Nachfolger wird zum aktuellen Knoten
			p = p->nachfolger;
        }
    }
};

int main() {
    // Beispielsätze
	Film film("Star Wars",1977,"Science-Fiction"), film1("Iron Man 3",2013,"Action"), film2("Toy Story",1995,"Animation");

	FilmListe filme;
	filme.hinzufuegen(film);
	filme.hinzufuegen(film1);
	filme.hinzufuegen(film2);
	
	filme.elementeAnzeigen();

	filme.loeschen();

    return 0;
}
